import 'package:apex_flutter_plugin/api/api.dart';
import 'package:apex_flutter_plugin/api/config.dart';
import 'package:apex_flutter_plugin/api/request.dart';
import 'package:apex_flutter_plugin/api/response.dart';
import 'package:apexid/fingerprint.dart';
import 'package:apexid/res/res_img.dart';
import 'package:apexid/router.dart';
import 'package:apexid/screens/settings_screen.dart';
import 'package:apexid/ui/fontManager.dart';
import 'package:apexid/utils/config.dart';
import 'package:apexid/utils/language.dart';
import 'package:apexid/utils/request.dart';
import 'package:apexid/utils/storage_util.dart';
import 'package:apexid/widgets/widget_load.dart';
import 'package:bot_toast/bot_toast.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:apexid/providers/login_notifier.dart';
import 'package:google_fonts/google_fonts.dart';
import 'package:marquee_widget/marquee_widget.dart';
import 'package:provider/provider.dart';
import 'package:provider/single_child_widget.dart';

class ApexId {
  static final ApexId _singleton = ApexId._();

  factory ApexId() => _singleton;

  ApexId._();

  static ApexId get shared => _singleton;

  Function(String accessKey, BuildContext context) onLogin;
  Function() onBack;
  String apiKey;
  BuildContext context;

  /// Initialize SDK in initState()
  initialize({@required String developerKey}) {
    apiKey = developerKey ?? "Null";
    WidgetsFlutterBinding.ensureInitialized();
    LicenseRegistry.addLicense(() async* {
      final license =
          await rootBundle.loadString('assets/google_fonts/OFL.txt');
      yield LicenseEntryWithLineBreaks(['google_fonts'], license);
    });
    StorageUtil.getInstance();
  }

  /// Get logged in accessKey (returned accessKey if exist or null for does not exist)
  String getAccessKey() {
    return StorageUtil.getString("access_key", defValue: null);
  }

  /// Logout from apexId
  Future<bool> logoutFromApexId({String developerKey}) async {
    if(developerKey != null) apiKey = developerKey;
    Response resp = await Api.shared.makeRequest(IsolateModel(
        Request({
          "fingerprint": await FingerPrint().get(),
          "action": RequestActions.logout,
          "api_key": apiKey,
          "access_key": StorageUtil.getString("access_key"),
          "lang": "en"
        }),
        url: Config.apiUrl));
    if (resp.result.params['ok']) {
      StorageUtil.putString("access_key", "null");
      return true;
    } else {
      return false;
    }
  }

  /// Use this to create login button and config the functions
  Widget apexLoginButton(
      {@required Function(String accessKey, BuildContext context) onLoginMethod,
      @required Widget child,
      BuildContext cntxt}) {
    context = cntxt;
    onLogin = onLoginMethod;
    return child;
  }

  Widget apexSettingButton({
    @required Widget child,
    @required Function() onBackPressed,
  }) {
    onBack = onBackPressed;
    return child;
  }
}

Future<Null> registerStorage() async {
  await StorageUtil.getInstance();
}

class ApexIdApp extends StatefulWidget {
  final Function(String accessKey) onLoginMethod;

  const ApexIdApp({Key key, this.onLoginMethod}) : super(key: key);

  // This widget is the root of your application.
  @override
  _ApexIdAppState createState() => _ApexIdAppState();
}

class _ApexIdAppState extends State<ApexIdApp> with WidgetLoadMixin {
  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: <SingleChildWidget>[
        ChangeNotifierProvider<LoginNotifier>(
          create: (_) => LoginNotifier(),
          lazy: true,
        ),
      ],
      child: Directionality(
        textDirection: StorageUtil.getString("lang", defValue: "en") == "en"
            ? TextDirection.ltr
            : TextDirection.rtl,
        child: MaterialApp(
          builder: BotToastInit(),
          title: 'Apex ID',
          navigatorObservers: [BotToastNavigatorObserver()],
          theme: ThemeData(
            primarySwatch: Colors.amber,
            visualDensity: VisualDensity.adaptivePlatformDensity,
          ),
          onGenerateRoute: ApexIdRouter.generateRoute,
          initialRoute: ApexIdRouter.splashRoute,
        ),
      ),
    );
  }

  void configApp() {
    Future.delayed(Duration(milliseconds: 200), () async {
      if (StorageUtil.getString("lang", defValue: "null").isEmpty) {
        StorageUtil.putString("lang", "en");
      }
      var defaultParam;
      FontManager(StorageUtil.getString("lang", defValue: "en"));
      if (StorageUtil.getString("access_key", defValue: "null") == "null") {
        defaultParam = {
          'fingerprint': await FingerPrint().get(),
          'api_key': ApexId.shared.apiKey,
          'lang': StorageUtil.getString("lang", defValue: "en")
        };
      } else {
        defaultParam = {
          'fingerprint': await FingerPrint().get(),
          'api_key': ApexId.shared.apiKey,
          'lang': StorageUtil.getString("lang", defValue: "en"),
          "access_key": StorageUtil.getString("access_key"),
        };
      }
      Api.shared.config = ApiConfig(Config.apiUrl,
          encrypt: false, debugMode: true, defaultParameters: defaultParam);
    });
  }

  @override
  void onLoad(BuildContext context) {
    configApp();
  }
}


class ApexIdSetting extends StatefulWidget {

  const ApexIdSetting({Key key}) : super(key: key);

  @override
  _ApexIdSettingState createState() => _ApexIdSettingState();
}

class _ApexIdSettingState extends State<ApexIdSetting>{
  @override
  void initState() {
    registerStorage();
    super.initState();
  }
  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: <SingleChildWidget>[
        ChangeNotifierProvider<LoginNotifier>(
          create: (_) => LoginNotifier(),
          lazy: true,
        ),
      ],
      child: MaterialApp(
        builder: BotToastInit(),
        navigatorObservers: [BotToastNavigatorObserver()],
        title: "Apex ID Settings",
        home: ApexIdSettingScreen(),
        theme: ThemeData(
          primarySwatch: Colors.amber,
          visualDensity: VisualDensity.adaptivePlatformDensity,
        ),
      ) ,
    );
  }
}
